@@ -1,4 +1,4 @@
-# Copyright 2019 The Keras Tuner Authors
+# Copyright 2019 The KerasTuner Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,11 @@ import tensorflow.keras as keras
 from tensorflow.keras import backend
 from tensorflow.keras import layers
 
+from keras_tuner.api_export import keras_tuner_export
 from keras_tuner.engine import hypermodel
 
 
+@keras_tuner_export("keras_tuner.applications.HyperResNet")
 class HyperResNet(hypermodel.HyperModel):
     """A ResNet hypermodel.
 
@@ -50,10 +52,9 @@ class HyperResNet(hypermodel.HyperModel):
         input_shape=None,
         input_tensor=None,
         classes=None,
-        **kwargs
+        **kwargs,
     ):
-
-        super(HyperResNet, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         if include_top and classes is None:
             raise ValueError("You must specify `classes` when " "`include_top=True`")
 
@@ -73,8 +74,8 @@ class HyperResNet(hypermodel.HyperModel):
         conv4_depth = hp.Choice("conv4_depth", [6, 23, 36])
 
         # Version-conditional fixed parameters
-        preact = True if version == "v2" else False
-        use_bias = False if version == "next" else True
+        preact = version == "v2"
+        use_bias = version != "next"
 
         # Model definition.
         bn_axis = 3 if backend.image_data_format() == "channels_last" else 1
@@ -89,7 +90,7 @@ class HyperResNet(hypermodel.HyperModel):
         # Initial conv2d block.
         x = layers.ZeroPadding2D(padding=((3, 3), (3, 3)), name="conv1_pad")(x)
         x = layers.Conv2D(64, 7, strides=2, use_bias=use_bias, name="conv1_conv")(x)
-        if preact is False:
+        if not preact:
             x = layers.BatchNormalization(
                 axis=bn_axis, epsilon=1.001e-5, name="conv1_bn"
             )(x)
@@ -115,7 +116,7 @@ class HyperResNet(hypermodel.HyperModel):
             x = stack3(x, 1024, 3, stride1=1, name="conv5")
 
         # Top of the model.
-        if preact is True:
+        if preact:
             x = layers.BatchNormalization(
                 axis=bn_axis, epsilon=1.001e-5, name="post_bn"
             )(x)
@@ -127,24 +128,23 @@ class HyperResNet(hypermodel.HyperModel):
         elif pooling == "max":
             x = layers.GlobalMaxPooling2D(name="max_pool")(x)
 
-        if self.include_top:
-            x = layers.Dense(self.classes, activation="softmax", name="probs")(x)
-            model = keras.Model(inputs, x, name="ResNet")
-            optimizer_name = hp.Choice(
-                "optimizer", ["adam", "rmsprop", "sgd"], default="adam"
-            )
-            optimizer = keras.optimizers.get(optimizer_name)
-            optimizer.learning_rate = hp.Choice(
-                "learning_rate", [0.1, 0.01, 0.001], default=0.01
-            )
-            model.compile(
-                optimizer=optimizer,
-                loss="categorical_crossentropy",
-                metrics=["accuracy"],
-            )
-            return model
-        else:
+        if not self.include_top:
             return keras.Model(inputs, x, name="ResNet")
+        x = layers.Dense(self.classes, activation="softmax", name="probs")(x)
+        model = keras.Model(inputs, x, name="ResNet")
+        optimizer_name = hp.Choice(
+            "optimizer", ["adam", "rmsprop", "sgd"], default="adam"
+        )
+        optimizer = keras.optimizers.get(optimizer_name)
+        optimizer.learning_rate = hp.Choice(
+            "learning_rate", [0.1, 0.01, 0.001], default=0.01
+        )
+        model.compile(
+            optimizer=optimizer,
+            loss="categorical_crossentropy",
+            metrics=["accuracy"],
+        )
+        return model
 
 
 def block1(x, filters, kernel_size=3, stride=1, conv_shortcut=True, name=None):
@@ -166,33 +166,38 @@ def block1(x, filters, kernel_size=3, stride=1, conv_shortcut=True, name=None):
 
     if conv_shortcut is True:
         shortcut = layers.Conv2D(
-            4 * filters, 1, strides=stride, name=name + "_0_conv"
+            4 * filters, 1, strides=stride, name=f"{name}_0_conv"
         )(x)
+
         shortcut = layers.BatchNormalization(
-            axis=bn_axis, epsilon=1.001e-5, name=name + "_0_bn"
+            axis=bn_axis, epsilon=1.001e-5, name=f"{name}_0_bn"
         )(shortcut)
+
     else:
         shortcut = x
 
-    x = layers.Conv2D(filters, 1, strides=stride, name=name + "_1_conv")(x)
+    x = layers.Conv2D(filters, 1, strides=stride, name=f"{name}_1_conv")(x)
     x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_1_bn"
-    )(x)
-    x = layers.Activation("relu", name=name + "_1_relu")(x)
-
-    x = layers.Conv2D(filters, kernel_size, padding="same", name=name + "_2_conv")(x)
-    x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_2_bn"
-    )(x)
-    x = layers.Activation("relu", name=name + "_2_relu")(x)
-
-    x = layers.Conv2D(4 * filters, 1, name=name + "_3_conv")(x)
-    x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_3_bn"
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_1_bn"
     )(x)
 
-    x = layers.Add(name=name + "_add")([shortcut, x])
-    x = layers.Activation("relu", name=name + "_out")(x)
+    x = layers.Activation("relu", name=f"{name}_1_relu")(x)
+
+    x = layers.Conv2D(filters, kernel_size, padding="same", name=f"{name}_2_conv")(x)
+
+    x = layers.BatchNormalization(
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_2_bn"
+    )(x)
+
+    x = layers.Activation("relu", name=f"{name}_2_relu")(x)
+
+    x = layers.Conv2D(4 * filters, 1, name=f"{name}_3_conv")(x)
+    x = layers.BatchNormalization(
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_3_bn"
+    )(x)
+
+    x = layers.Add(name=f"{name}_add")([shortcut, x])
+    x = layers.Activation("relu", name=f"{name}_out")(x)
     return x
 
 
@@ -209,9 +214,9 @@ def stack1(x, filters, blocks, stride1=2, name=None):
     Returns:
         Output tensor for the stacked blocks.
     """
-    x = block1(x, filters, stride=stride1, name=name + "_block1")
+    x = block1(x, filters, stride=stride1, name=f"{name}_block1")
     for i in range(2, blocks + 1):
-        x = block1(x, filters, conv_shortcut=False, name=name + "_block" + str(i))
+        x = block1(x, filters, conv_shortcut=False, name=f"{name}_block{str(i)}")
     return x
 
 
@@ -233,36 +238,42 @@ def block2(x, filters, kernel_size=3, stride=1, conv_shortcut=False, name=None):
     bn_axis = 3 if backend.image_data_format() == "channels_last" else 1
 
     preact = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_preact_bn"
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_preact_bn"
     )(x)
-    preact = layers.Activation("relu", name=name + "_preact_relu")(preact)
+
+    preact = layers.Activation("relu", name=f"{name}_preact_relu")(preact)
 
     if conv_shortcut is True:
         shortcut = layers.Conv2D(
-            4 * filters, 1, strides=stride, name=name + "_0_conv"
+            4 * filters, 1, strides=stride, name=f"{name}_0_conv"
         )(preact)
+
     else:
         shortcut = layers.MaxPooling2D(1, strides=stride)(x) if stride > 1 else x
 
-    x = layers.Conv2D(filters, 1, strides=1, use_bias=False, name=name + "_1_conv")(
+    x = layers.Conv2D(filters, 1, strides=1, use_bias=False, name=f"{name}_1_conv")(
         preact
     )
-    x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_1_bn"
-    )(x)
-    x = layers.Activation("relu", name=name + "_1_relu")(x)
 
-    x = layers.ZeroPadding2D(padding=((1, 1), (1, 1)), name=name + "_2_pad")(x)
+    x = layers.BatchNormalization(
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_1_bn"
+    )(x)
+
+    x = layers.Activation("relu", name=f"{name}_1_relu")(x)
+
+    x = layers.ZeroPadding2D(padding=((1, 1), (1, 1)), name=f"{name}_2_pad")(x)
     x = layers.Conv2D(
-        filters, kernel_size, strides=stride, use_bias=False, name=name + "_2_conv"
+        filters, kernel_size, strides=stride, use_bias=False, name=f"{name}_2_conv"
     )(x)
-    x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_2_bn"
-    )(x)
-    x = layers.Activation("relu", name=name + "_2_relu")(x)
 
-    x = layers.Conv2D(4 * filters, 1, name=name + "_3_conv")(x)
-    x = layers.Add(name=name + "_out")([shortcut, x])
+    x = layers.BatchNormalization(
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_2_bn"
+    )(x)
+
+    x = layers.Activation("relu", name=f"{name}_2_relu")(x)
+
+    x = layers.Conv2D(4 * filters, 1, name=f"{name}_3_conv")(x)
+    x = layers.Add(name=f"{name}_out")([shortcut, x])
     return x
 
 
@@ -279,10 +290,10 @@ def stack2(x, filters, blocks, stride1=2, name=None):
     Returns:
         Output tensor for the stacked blocks.
     """
-    x = block2(x, filters, conv_shortcut=True, name=name + "_block1")
+    x = block2(x, filters, conv_shortcut=True, name=f"{name}_block1")
     for i in range(2, blocks):
-        x = block2(x, filters, name=name + "_block" + str(i))
-    x = block2(x, filters, stride=stride1, name=name + "_block" + str(blocks))
+        x = block2(x, filters, name=f"{name}_block{str(i)}")
+    x = block2(x, filters, stride=stride1, name=f"{name}_block{str(blocks)}")
     return x
 
 
@@ -312,57 +323,61 @@ def block3(
             1,
             strides=stride,
             use_bias=False,
-            name=name + "_0_conv",
+            name=f"{name}_0_conv",
         )(x)
+
         shortcut = layers.BatchNormalization(
-            axis=bn_axis, epsilon=1.001e-5, name=name + "_0_bn"
+            axis=bn_axis, epsilon=1.001e-5, name=f"{name}_0_bn"
         )(shortcut)
+
     else:
         shortcut = x
 
-    x = layers.Conv2D(filters, 1, use_bias=False, name=name + "_1_conv")(x)
+    x = layers.Conv2D(filters, 1, use_bias=False, name=f"{name}_1_conv")(x)
     x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_1_bn"
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_1_bn"
     )(x)
-    x = layers.Activation("relu", name=name + "_1_relu")(x)
+
+    x = layers.Activation("relu", name=f"{name}_1_relu")(x)
 
     c = filters // groups
-    x = layers.ZeroPadding2D(padding=((1, 1), (1, 1)), name=name + "_2_pad")(x)
+    x = layers.ZeroPadding2D(padding=((1, 1), (1, 1)), name=f"{name}_2_pad")(x)
     x = layers.DepthwiseConv2D(
         kernel_size,
         strides=stride,
         depth_multiplier=c,
         use_bias=False,
-        name=name + "_2_conv",
+        name=f"{name}_2_conv",
     )(x)
+
     x_shape = backend.int_shape(x)[1:-1]
     x = layers.Reshape(x_shape + (groups, c, c))(x)
     output_shape = x_shape + (groups, c) if backend.backend() == "theano" else None
 
     x = layers.Lambda(
-        lambda x: sum([x[:, :, :, :, i] for i in range(c)]),
+        lambda x: sum(x[:, :, :, :, i] for i in range(c)),
         output_shape=output_shape,
-        name=name + "_2_reduce",
+        name=f"{name}_2_reduce",
     )(x)
 
     x = layers.Reshape(x_shape + (filters,))(x)
 
     x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_2_bn"
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_2_bn"
     )(x)
 
-    x = layers.Activation("relu", name=name + "_2_relu")(x)
+    x = layers.Activation("relu", name=f"{name}_2_relu")(x)
 
     x = layers.Conv2D(
-        (64 // groups) * filters, 1, use_bias=False, name=name + "_3_conv"
+        (64 // groups) * filters, 1, use_bias=False, name=f"{name}_3_conv"
     )(x)
 
     x = layers.BatchNormalization(
-        axis=bn_axis, epsilon=1.001e-5, name=name + "_3_bn"
+        axis=bn_axis, epsilon=1.001e-5, name=f"{name}_3_bn"
     )(x)
 
-    x = layers.Add(name=name + "_add")([shortcut, x])
-    x = layers.Activation("relu", name=name + "_out")(x)
+    x = layers.Add(name=f"{name}_add")([shortcut, x])
+    x = layers.Activation("relu", name=f"{name}_out")(x)
     return x
 
 
@@ -380,7 +395,7 @@ def stack3(x, filters, blocks, stride1=2, groups=32, name=None):
     Returns:
         Output tensor for the stacked blocks.
     """
-    x = block3(x, filters, stride=stride1, groups=groups, name=name + "_block1")
+    x = block3(x, filters, stride=stride1, groups=groups, name=f"{name}_block1")
 
     for i in range(2, blocks + 1):
         x = block3(
@@ -388,6 +403,7 @@ def stack3(x, filters, blocks, stride1=2, groups=32, name=None):
             filters,
             groups=groups,
             conv_shortcut=False,
-            name=name + "_block" + str(i),
+            name=f"{name}_block{str(i)}",
         )
+
     return x
