@@ -1,4 +1,4 @@
-# Copyright 2019 The Keras Tuner Authors
+# Copyright 2019 The KerasTuner Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,11 @@ import tensorflow as tf
 import tensorflow.keras as keras
 from tensorflow.keras import layers
 
+from keras_tuner.api_export import keras_tuner_export
 from keras_tuner.engine import hypermodel
 
 
+@keras_tuner_export("keras_tuner.applications.HyperXception")
 class HyperXception(hypermodel.HyperModel):
     """An Xception hypermodel.
 
@@ -47,9 +49,9 @@ class HyperXception(hypermodel.HyperModel):
         input_shape=None,
         input_tensor=None,
         classes=None,
-        **kwargs
+        **kwargs,
     ):
-        super(HyperXception, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         if include_top and classes is None:
             raise ValueError("You must specify `classes` when " "`include_top=True`")
 
@@ -106,32 +108,31 @@ class HyperXception(hypermodel.HyperModel):
         else:
             x = layers.GlobalMaxPooling2D()(x)
 
-        if self.include_top:
-            # Dense
-            num_dense_layers = hp.Int("num_dense_layers", 1, 3)
-            dropout_rate = hp.Float("dropout_rate", 0.0, 0.6, step=0.1, default=0.5)
-            dense_use_bn = hp.Choice("dense_use_bn", [True, False])
-            for _ in range(num_dense_layers):
-                x = dense(
-                    x,
-                    self.classes,
-                    activation=activation,
-                    batchnorm=dense_use_bn,
-                    dropout_rate=dropout_rate,
-                )
-            output = layers.Dense(self.classes, activation="softmax")(x)
-            model = keras.Model(inputs, output, name="Xception")
-
-            model.compile(
-                optimizer=keras.optimizers.Adam(
-                    hp.Choice("learning_rate", [1e-3, 1e-4, 1e-5])
-                ),
-                loss="categorical_crossentropy",
-                metrics=["accuracy"],
-            )
-            return model
-        else:
+        if not self.include_top:
             return keras.Model(inputs, x, name="Xception")
+        # Dense
+        num_dense_layers = hp.Int("num_dense_layers", 1, 3)
+        dropout_rate = hp.Float("dropout_rate", 0.0, 0.6, step=0.1, default=0.5)
+        dense_use_bn = hp.Choice("dense_use_bn", [True, False])
+        for _ in range(num_dense_layers):
+            x = dense(
+                x,
+                self.classes,
+                activation=activation,
+                batchnorm=dense_use_bn,
+                dropout_rate=dropout_rate,
+            )
+        output = layers.Dense(self.classes, activation="softmax")(x)
+        model = keras.Model(inputs, output, name="Xception")
+
+        model.compile(
+            optimizer=keras.optimizers.Adam(
+                hp.Choice("learning_rate", [1e-3, 1e-4, 1e-5])
+            ),
+            loss="categorical_crossentropy",
+            metrics=["accuracy"],
+        )
+        return model
 
 
 def sep_conv(x, num_filters, kernel_size=(3, 3), activation="relu"):
@@ -150,7 +151,7 @@ def sep_conv(x, num_filters, kernel_size=(3, 3), activation="relu"):
         x = layers.BatchNormalization()(x)
         x = layers.Activation("relu")(x)
     else:
-        ValueError("Unknown activation function: %s" % (activation,))
+        ValueError(f"Unknown activation function: {activation}")
     return x
 
 
@@ -200,7 +201,7 @@ def conv(x, num_filters, kernel_size=(3, 3), activation="relu", strides=(2, 2)):
         x = layers.BatchNormalization()(x)
         x = layers.Activation("relu")(x)
     else:
-        msg = "Unknown activation function: %s" % activation
+        msg = f"Unknown activation function: {activation}"
         ValueError(msg)
     return x
 
@@ -222,6 +223,6 @@ def dense(x, dims, activation="relu", batchnorm=True, dropout_rate=0):
         if dropout_rate:
             x = layers.Dropout(dropout_rate)(x)
     else:
-        msg = "Unknown activation function: %s" % activation
+        msg = f"Unknown activation function: {activation}"
         ValueError(msg)
     return x
