@@ -23,7 +23,7 @@ from keras_tuner.protos import service_pb2
 from keras_tuner.protos import service_pb2_grpc
 
 
-class OracleClient(object):
+class OracleClient:
     """Wraps an `Oracle` on a worker to send requests to the chief."""
 
     def __init__(self, oracle):
@@ -31,7 +31,7 @@ class OracleClient(object):
 
         ip_addr = os.environ["KERASTUNER_ORACLE_IP"]
         port = os.environ["KERASTUNER_ORACLE_PORT"]
-        channel = grpc.insecure_channel("{}:{}".format(ip_addr, port))
+        channel = grpc.insecure_channel(f"{ip_addr}:{port}")
         self.stub = service_pb2_grpc.OracleStub(channel)
         self.tuner_id = os.environ["KERASTUNER_TUNER_ID"]
 
@@ -46,13 +46,10 @@ class OracleClient(object):
             "max_trials",
             "allow_new_entries",
             "tune_new_entries",
-            "trials",
         }
         if name in whitelisted_attrs:
             return getattr(self._oracle, name)
-        raise AttributeError(
-            '`OracleClient` object has no attribute "{}"'.format(name)
-        )
+        raise AttributeError(f'`OracleClient` object has no attribute "{name}"')
 
     def get_space(self):
         response = self.stub.GetSpace(
@@ -85,15 +82,13 @@ class OracleClient(object):
                 wait_for_ready=True,
             )
             if not self.multi_worker:
-                return trial_module._convert_trial_status_to_str(response.status)
-            return "RUNNING"
+                return trial_module.TrialStatus.from_proto(response.status)
         return "RUNNING"
 
-    def end_trial(self, trial_id, status="COMPLETED"):
+    def end_trial(self, trial):
         if self.should_report:
-            status = trial_module._convert_trial_status_to_proto(status)
             self.stub.EndTrial(
-                service_pb2.EndTrialRequest(trial_id=trial_id, status=status),
+                service_pb2.EndTrialRequest(trial=trial.to_proto()),
                 wait_for_ready=True,
             )
 
