@@ -1,3 +1,17 @@
+# Copyright 2019 The KerasTuner Authors
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     https://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 "Cloud service related functionality."
 
 
@@ -13,7 +27,7 @@ AUTH_ERROR = 3
 UPLOAD_ERROR = 4
 
 
-class Logger(object):
+class Logger:
     def register_tuner(self, tuner_state):
         """Informs the logger that a new search is starting."""
         raise NotImplementedError
@@ -37,22 +51,20 @@ def url_join(*parts):
 def send_to_backend(url, data, key):
     response = requests.post(url, headers={"X-AUTH": key}, json=data)
 
-    if not response.ok:
-        try:
-            response_json = response.json()
-        except json.decoder.JSONDecodeError:
-            print("Cloud service down -- data not uploaded: %s" % response.text)
-            return CONNECT_ERROR
-
-        if response_json["status"] == "Unauthorized":
-            print("Invalid backend API key.")
-            return AUTH_ERROR
-        else:
-            print("Warning! Cloud service upload failed: %s" % response.text)
-            return UPLOAD_ERROR
-        return ERROR
-    else:
+    if response.ok:
         return OK
+    try:
+        response_json = response.json()
+    except json.decoder.JSONDecodeError:
+        print(f"Cloud service down -- data not uploaded: {response.text}")
+        return CONNECT_ERROR
+
+    if response_json["status"] == "Unauthorized":
+        print("Invalid backend API key.")
+        return AUTH_ERROR
+    else:
+        print(f"Warning! Cloud service upload failed: {response.text}")
+        return UPLOAD_ERROR
 
 
 class CloudLogger(Logger):
