@@ -1,4 +1,4 @@
-# Copyright 2019 The Keras Tuner Authors
+# Copyright 2019 The KerasTuner Authors
 #
 # Licensed under the Apache License, Version 2.0 (the 'License');
 # you may not use this file except in compliance with the License.
@@ -12,18 +12,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+import inspect
 
 import numpy as np
 import six
 from tensorflow import keras
 
-from ..protos import keras_tuner_pb2
+from keras_tuner.protos import keras_tuner_pb2
 
 
-class MetricObservation(object):
+class MetricObservation:
+    """Metric value at a given step of training across multiple executions.
+
+    If the model is trained multiple
+    times (multiple executions), KerasTuner records the value of each
+    metric at each training step. These values are aggregated
+    over multiple executions into a list where each value corresponds
+    to one execution.
+
+    Args:
+        value: Float or a list of floats. The evaluated metric values.
+        step: Int. The step of the evaluation, for example, the epoch number.
+    """
+
     def __init__(self, value, step):
         if not isinstance(value, list):
             value = [value]
@@ -46,12 +57,14 @@ class MetricObservation(object):
         return cls(**config)
 
     def __eq__(self, other):
-        if not isinstance(other, MetricObservation):
-            return False
-        return other.value == self.value and other.step == self.step
+        return (
+            other.value == self.value and other.step == self.step
+            if isinstance(other, MetricObservation)
+            else False
+        )
 
     def __repr__(self):
-        return "MetricObservation(value={}, step={})".format(self.value, self.step)
+        return f"MetricObservation(value={self.value}, step={self.step})"
 
     def to_proto(self):
         return keras_tuner_pb2.MetricObservation(value=self.value, step=self.step)
@@ -61,7 +74,16 @@ class MetricObservation(object):
         return cls(value=list(proto.value), step=proto.step)
 
 
-class MetricHistory(object):
+class MetricHistory:
+    """Record of multiple executions of a single metric.
+
+    It contains a collection of `MetricObservation` instances.
+
+    Args:
+        direction: String. The direction of the metric to optimize. The value
+            should be "min" or "max".
+    """
+
     def __init__(self, direction="min"):
         if direction not in {"min", "max"}:
             raise ValueError(
@@ -69,6 +91,7 @@ class MetricHistory(object):
                 '{"min", "max"}, but got: %s' % (direction,)
             )
         self.direction = direction
+        # Mapping step to `MetricObservation`.
         self._observations = {}
 
     def update(self, value, step):
@@ -78,12 +101,10 @@ class MetricHistory(object):
             self._observations[step] = MetricObservation(value, step=step)
 
     def get_best_value(self):
-        values = list(obs.mean() for obs in self._observations.values())
+        values = [obs.mean() for obs in self._observations.values()]
         if not values:
             return None
-        if self.direction == "min":
-            return np.nanmin(values)
-        return np.nanmax(values)
+        return np.nanmin(values) if self.direction == "min" else np.nanmax(values)
 
     def get_best_step(self):
         best_value = self.get_best_value()
@@ -103,16 +124,18 @@ class MetricHistory(object):
     def get_statistics(self):
         history = self.get_history()
         history_values = [obs.mean() for obs in history]
-        if not len(history_values):
-            return {}
-        return {
-            "min": float(np.nanmin(history_values)),
-            "max": float(np.nanmax(history_values)),
-            "mean": float(np.nanmean(history_values)),
-            "median": float(np.nanmedian(history_values)),
-            "var": float(np.nanvar(history_values)),
-            "std": float(np.nanstd(history_values)),
-        }
+        return (
+            {
+                "min": float(np.nanmin(history_values)),
+                "max": float(np.nanmax(history_values)),
+                "mean": float(np.nanmean(history_values)),
+                "median": float(np.nanmedian(history_values)),
+                "var": float(np.nanvar(history_values)),
+                "std": float(np.nanstd(history_values)),
+            }
+            if len(history_values)
+            else {}
+        )
 
     def get_last_value(self):
         history = self.get_history()
@@ -123,9 +146,11 @@ class MetricHistory(object):
             return None
 
     def get_config(self):
-        config = {}
-        config["direction"] = self.direction
-        config["observations"] = [obs.get_config() for obs in self.get_history()]
+        config = {
+            "direction": self.direction,
+            "observations": [obs.get_config() for obs in self.get_history()],
+        }
+
         return config
 
     @classmethod
@@ -152,7 +177,15 @@ class MetricHistory(object):
         return instance
 
 
-class MetricsTracker(object):
+class MetricsTracker:
+    """Record of the values of multiple executions of all metrics.
+
+    It contains `MetricHistory` instances for the metrics.
+
+    Args:
+        metrics: List of strings of the names of the metrics.
+    """
+
     def __init__(self, metrics=None):
         # str -> MetricHistory
         self.metrics = {}
@@ -168,13 +201,13 @@ class MetricsTracker(object):
 
     def register(self, name, direction=None):
         if self.exists(name):
-            raise ValueError("Metric already exists: %s" % (name,))
+            raise ValueError(f"Metric already exists: {name}")
         if direction is None:
             direction = infer_metric_direction(name)
-            if direction is None:
-                # Objective direction is handled separately, but
-                # non-objective direction defaults to min.
-                direction = "min"
+        if direction is None:
+            # Objective direction is handled separately, but
+            # non-objective direction defaults to min.
+            direction = "min"
         self.metrics[name] = MetricHistory(direction)
 
     def update(self, name, value, step=0):
@@ -255,7 +288,7 @@ class MetricsTracker(object):
 
     def _assert_exists(self, name):
         if name not in self.metrics:
-            raise ValueError("Unknown metric: %s" % (name,))
+            raise ValueError(f"Unknown metric: {name}")
 
 
 _MAX_METRICS = (
@@ -300,11 +333,27 @@ def infer_metric_direction(metric):
             return "max"
 
         try:
-            metric = keras.metrics.get(metric_name)
+            if (
+                "use_legacy_format"
+                in inspect.getargspec(keras.metrics.deserialize).args
+            ):
+                metric = keras.metrics.deserialize(
+                    metric_name, use_legacy_format=True
+                )
+            else:
+                metric = keras.metrics.deserialize(metric_name)
         except ValueError:
             try:
-                metric = keras.losses.get(metric_name)
-            except:
+                if (
+                    "use_legacy_format"
+                    in inspect.getargspec(keras.losses.deserialize).args
+                ):
+                    metric = keras.losses.deserialize(
+                        metric_name, use_legacy_format=True
+                    )
+                else:
+                    metric = keras.losses.deserialize(metric_name)
+            except Exception:
                 # Direction can't be inferred.
                 return None
 
