@@ -24,10 +24,12 @@ from tensorboard.plugins.hparams import api as hparams_api
 from tensorflow import keras
 
 from keras_tuner import errors
+from keras_tuner.api_export import keras_tuner_export
 from keras_tuner.engine import base_tuner
 from keras_tuner.engine import tuner_utils
 
 
+@keras_tuner_export(["keras_tuner.Tuner", "keras_tuner.tuners.Tuner"])
 class Tuner(base_tuner.BaseTuner):
     """Tuner class for Keras models.
 
