@@ -1,4 +1,4 @@
-# Copyright 2019 The Keras Tuner Authors
+# Copyright 2019 The KerasTuner Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,12 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import math
-import random
 
-from ..engine import multi_execution_tuner
-from ..engine import oracle as oracle_module
+from keras_tuner.api_export import keras_tuner_export
+from keras_tuner.engine import oracle as oracle_module
+from keras_tuner.engine import tuner as tuner_module
 
 
+@keras_tuner_export("keras_tuner.oracles.HyperbandOracle")
 class HyperbandOracle(oracle_module.Oracle):
     """Oracle class for Hyperband.
 
@@ -57,13 +58,19 @@ class HyperbandOracle(oracle_module.Oracle):
     ```
 
     Args:
-        objective: A string or `keras_tuner.Objective` instance. If a string,
-            the direction of the optimization (min or max) will be inferred.
+        objective: A string, `keras_tuner.Objective` instance, or a list of
+            `keras_tuner.Objective`s and strings. If a string, the direction of
+            the optimization (min or max) will be inferred. If a list of
+            `keras_tuner.Objective`, we will minimize the sum of all the
+            objectives to minimize subtracting the sum of all the objectives to
+            maximize. The `objective` argument is optional when
+            `Tuner.run_trial()` or `HyperModel.fit()` returns a single float as
+            the objective to minimize.
         max_epochs: Integer, the maximum number of epochs to train one model.
             It is recommended to set this to a value slightly higher than the
             expected epochs to convergence for your largest Model, and to use
             early stopping during training (for example, via
-            `tf.keras.callbacks.EarlyStopping`).
+            `tf.keras.callbacks.EarlyStopping`). Defaults to 100.
         factor: Integer, the reduction factor for the number of epochs
             and number of models for each bracket. Defaults to 3.
         hyperband_iterations: Integer, at least 1, the number of times to
@@ -84,25 +91,35 @@ class HyperbandOracle(oracle_module.Oracle):
         allow_new_entries: Boolean, whether the hypermodel is allowed to
             request hyperparameter entries not listed in `hyperparameters`.
             Defaults to True.
+        max_retries_per_trial: Integer. Defaults to 0. The maximum number of
+            times to retry a `Trial` if the trial crashed or the results are invalid.
+        max_consecutive_failed_trials: Integer. Defaults to 3. The maximum number of
+            consecutive failed `Trial`s. When this number is reached, the search
+            will be stopped. A `Trial` is marked as failed when none of the
+            retries succeeded.
     """
 
     def __init__(
         self,
-        objective,
-        max_epochs,
+        objective=None,
+        max_epochs=100,
         factor=3,
         hyperband_iterations=1,
         seed=None,
         hyperparameters=None,
         allow_new_entries=True,
         tune_new_entries=True,
+        max_retries_per_trial=0,
+        max_consecutive_failed_trials=3,
     ):
-        super(HyperbandOracle, self).__init__(
+        super().__init__(
             objective=objective,
             hyperparameters=hyperparameters,
             allow_new_entries=allow_new_entries,
             tune_new_entries=tune_new_entries,
             seed=seed,
+            max_retries_per_trial=max_retries_per_trial,
+            max_consecutive_failed_trials=max_consecutive_failed_trials,
         )
         if factor < 2:
             raise ValueError("factor needs to be a int larger than 1.")
@@ -114,11 +131,6 @@ class HyperbandOracle(oracle_module.Oracle):
         self.min_epochs = 1
         self.factor = factor
 
-        self.seed = seed or random.randint(1, 10000)
-        self._max_collisions = 20
-        self._seed_state = self.seed
-        self._tried_so_far = set()
-
         self._current_iteration = 0
         # Start with most aggressively halving bracket.
         self._current_bracket = self._get_num_brackets() - 1
@@ -127,6 +139,19 @@ class HyperbandOracle(oracle_module.Oracle):
         self._start_new_bracket()
 
     def populate_space(self, trial_id):
+        """Fill the hyperparameter space with values.
+
+        Args:
+            trial_id: A string, the ID for this Trial.
+
+        Returns:
+            A dictionary with keys "values" and "status", where "values" is
+            a mapping of parameter names to suggested values, and "status"
+            should be one of "RUNNING" (the trial can start normally), "IDLE"
+            (the oracle is waiting on something and cannot create a trial), or
+            "STOPPED" (the oracle has finished searching and no new trial should
+            be created).
+        """
         self._remove_completed_brackets()
 
         for bracket in self._brackets:
@@ -136,46 +161,43 @@ class HyperbandOracle(oracle_module.Oracle):
             if len(rounds[0]) < self._get_size(bracket_num, round_num=0):
                 # Populate the initial random trials for this bracket.
                 return self._random_trial(trial_id, bracket)
-            else:
-                # Try to populate incomplete rounds for this bracket.
-                for round_num in range(1, len(rounds)):
-                    round_info = rounds[round_num]
-                    past_round_info = rounds[round_num - 1]
-                    size = self._get_size(bracket_num, round_num)
-                    past_size = self._get_size(bracket_num, round_num - 1)
+            # Try to populate incomplete rounds for this bracket.
+            for round_num in range(1, len(rounds)):
+                round_info = rounds[round_num]
+                past_round_info = rounds[round_num - 1]
+                size = self._get_size(bracket_num, round_num)
+                past_size = self._get_size(bracket_num, round_num - 1)
 
-                    # If more trials from the last round are ready than will be
-                    # thrown out, we can select the best to run for the next round.
-                    already_selected = [info["past_id"] for info in round_info]
-                    candidates = [
-                        self.trials[info["id"]]
-                        for info in past_round_info
-                        if info["id"] not in already_selected
-                    ]
-                    candidates = [t for t in candidates if t.status == "COMPLETED"]
-                    if len(candidates) > past_size - size:
-                        sorted_candidates = sorted(
-                            candidates,
-                            key=lambda t: t.score,
-                            reverse=self.objective.direction == "max",
-                        )
-                        best_trial = sorted_candidates[0]
+                # If more trials from the last round are ready than will be
+                # thrown out, we can select the best to run for the next round.
+                already_selected = [info["past_id"] for info in round_info]
+                candidates = [
+                    self.trials[info["id"]]
+                    for info in past_round_info
+                    if info["id"] not in already_selected
+                ]
+                candidates = [t for t in candidates if t.status == "COMPLETED"]
+                if len(candidates) > past_size - size:
+                    sorted_candidates = sorted(
+                        candidates,
+                        key=lambda t: t.score,
+                        reverse=self.objective.direction == "max",
+                    )
+                    best_trial = sorted_candidates[0]
 
-                        values = best_trial.hyperparameters.values.copy()
-                        values["tuner/trial_id"] = best_trial.trial_id
-                        values["tuner/epochs"] = self._get_epochs(
-                            bracket_num, round_num
-                        )
-                        values["tuner/initial_epoch"] = self._get_epochs(
-                            bracket_num, round_num - 1
-                        )
-                        values["tuner/bracket"] = self._current_bracket
-                        values["tuner/round"] = round_num
+                    values = best_trial.hyperparameters.values.copy()
+                    values["tuner/trial_id"] = best_trial.trial_id
+                    values["tuner/epochs"] = self._get_epochs(bracket_num, round_num)
+                    values["tuner/initial_epoch"] = self._get_epochs(
+                        bracket_num, round_num - 1
+                    )
+                    values["tuner/bracket"] = self._current_bracket
+                    values["tuner/round"] = round_num
 
-                        round_info.append(
-                            {"past_id": best_trial.trial_id, "id": trial_id}
-                        )
-                        return {"status": "RUNNING", "values": values}
+                    round_info.append(
+                        {"past_id": best_trial.trial_id, "id": trial_id}
+                    )
+                    return {"status": "RUNNING", "values": values}
 
         # This is reached if no trials from current brackets can be run.
 
@@ -187,9 +209,8 @@ class HyperbandOracle(oracle_module.Oracle):
             # Stop creating new brackets, but wait to complete other brackets.
             if self.ongoing_trials:
                 return {"status": "IDLE"}
-            else:
-                self._increment_bracket_num()
-                return {"status": "STOPPED"}
+            self._increment_bracket_num()
+            return {"status": "STOPPED"}
         # Create a new bracket.
         else:
             self._increment_bracket_num()
@@ -198,8 +219,7 @@ class HyperbandOracle(oracle_module.Oracle):
 
     def _start_new_bracket(self):
         rounds = []
-        for _ in range(self._get_num_rounds(self._current_bracket)):
-            rounds.append([])
+        rounds.extend([] for _ in range(self._get_num_rounds(self._current_bracket)))
         bracket = {"bracket_num": self._current_bracket, "rounds": rounds}
         self._brackets.append(bracket)
 
@@ -217,10 +237,7 @@ class HyperbandOracle(oracle_module.Oracle):
             bracket_num = bracket["bracket_num"]
             rounds = bracket["rounds"]
             last_round = len(rounds) - 1
-            if len(rounds[last_round]) == self._get_size(bracket_num, last_round):
-                # All trials have been created for the current bracket.
-                return False
-            return True
+            return len(rounds[last_round]) != self._get_size(bracket_num, last_round)
 
         self._brackets = list(filter(_bracket_is_incomplete, self._brackets))
 
@@ -265,7 +282,7 @@ class HyperbandOracle(oracle_module.Oracle):
         return brackets
 
     def get_state(self):
-        state = super(HyperbandOracle, self).get_state()
+        state = super().get_state()
         state.update(
             {
                 "hyperband_iterations": self.hyperband_iterations,
@@ -280,7 +297,7 @@ class HyperbandOracle(oracle_module.Oracle):
         return state
 
     def set_state(self, state):
-        super(HyperbandOracle, self).set_state(state)
+        super().set_state(state)
         self.hyperband_iterations = state["hyperband_iterations"]
         self.max_epochs = state["max_epochs"]
         self.min_epochs = state["min_epochs"]
@@ -290,7 +307,8 @@ class HyperbandOracle(oracle_module.Oracle):
         self._current_iteration = state["current_iteration"]
 
 
-class Hyperband(multi_execution_tuner.MultiExecutionTuner):
+@keras_tuner_export(["keras_tuner.Hyperband", "keras_tuner.tuners.Hyperband"])
+class Hyperband(tuner_module.Tuner):
     """Variation of HyperBand algorithm.
 
     Reference:
@@ -302,15 +320,23 @@ class Hyperband(multi_execution_tuner.MultiExecutionTuner):
 
 
     Args:
-        hypermodel: A `HyperModel` instance (or callable that takes
-            hyperparameters and returns a Model instance).
-        objective: A string or `keras_tuner.Objective` instance. If a string,
-            the direction of the optimization (min or max) will be inferred.
+        hypermodel: Instance of `HyperModel` class (or callable that takes
+            hyperparameters and returns a `Model` instance). It is optional
+            when `Tuner.run_trial()` is overriden and does not use
+            `self.hypermodel`.
+        objective: A string, `keras_tuner.Objective` instance, or a list of
+            `keras_tuner.Objective`s and strings. If a string, the direction of
+            the optimization (min or max) will be inferred. If a list of
+            `keras_tuner.Objective`, we will minimize the sum of all the
+            objectives to minimize subtracting the sum of all the objectives to
+            maximize. The `objective` argument is optional when
+            `Tuner.run_trial()` or `HyperModel.fit()` returns a single float as
+            the objective to minimize.
         max_epochs: Integer, the maximum number of epochs to train one model.
             It is recommended to set this to a value slightly higher than the
             expected epochs to convergence for your largest Model, and to use
             early stopping during training (for example, via
-            `tf.keras.callbacks.EarlyStopping`).
+            `tf.keras.callbacks.EarlyStopping`). Defaults to 100.
         factor: Integer, the reduction factor for the number of epochs
             and number of models for each bracket. Defaults to 3.
         hyperband_iterations: Integer, at least 1, the number of times to
@@ -331,21 +357,29 @@ class Hyperband(multi_execution_tuner.MultiExecutionTuner):
         allow_new_entries: Boolean, whether the hypermodel is allowed to
             request hyperparameter entries not listed in `hyperparameters`.
             Defaults to True.
+        max_retries_per_trial: Integer. Defaults to 0. The maximum number of
+            times to retry a `Trial` if the trial crashed or the results are invalid.
+        max_consecutive_failed_trials: Integer. Defaults to 3. The maximum number of
+            consecutive failed `Trial`s. When this number is reached, the search
+            will be stopped. A `Trial` is marked as failed when none of the
+            retries succeeded.
         **kwargs: Keyword arguments relevant to all `Tuner` subclasses.
             Please see the docstring for `Tuner`.
     """
 
     def __init__(
         self,
-        hypermodel,
-        objective,
-        max_epochs,
+        hypermodel=None,
+        objective=None,
+        max_epochs=100,
         factor=3,
         hyperband_iterations=1,
         seed=None,
         hyperparameters=None,
         tune_new_entries=True,
         allow_new_entries=True,
+        max_retries_per_trial=0,
+        max_consecutive_failed_trials=3,
         **kwargs
     ):
         oracle = HyperbandOracle(
@@ -357,27 +391,22 @@ class Hyperband(multi_execution_tuner.MultiExecutionTuner):
             hyperparameters=hyperparameters,
             tune_new_entries=tune_new_entries,
             allow_new_entries=allow_new_entries,
+            max_retries_per_trial=max_retries_per_trial,
+            max_consecutive_failed_trials=max_consecutive_failed_trials,
         )
-        super(Hyperband, self).__init__(
-            oracle=oracle, hypermodel=hypermodel, **kwargs
-        )
+        super().__init__(oracle=oracle, hypermodel=hypermodel, **kwargs)
 
     def run_trial(self, trial, *fit_args, **fit_kwargs):
         hp = trial.hyperparameters
         if "tuner/epochs" in hp.values:
             fit_kwargs["epochs"] = hp.values["tuner/epochs"]
             fit_kwargs["initial_epoch"] = hp.values["tuner/initial_epoch"]
-        super(Hyperband, self).run_trial(trial, *fit_args, **fit_kwargs)
+        return super().run_trial(trial, *fit_args, **fit_kwargs)
 
-    def _build_model(self, hp):
-        model = super(Hyperband, self)._build_model(hp)
+    def _build_hypermodel(self, hp):
+        model = super()._build_hypermodel(hp)
         if "tuner/trial_id" in hp.values:
             trial_id = hp.values["tuner/trial_id"]
-            history_trial = self.oracle.get_trial(trial_id)
             # Load best checkpoint from this trial.
-            model.load_weights(
-                self._get_checkpoint_fname(
-                    history_trial.trial_id, history_trial.best_step
-                )
-            )
+            model.load_weights(self._get_checkpoint_fname(trial_id))
         return model
