@@ -1,4 +1,4 @@
-# Copyright 2019 The Keras Tuner Authors
+# Copyright 2019 The KerasTuner Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,25 +14,30 @@
 
 "Basic random search tuner."
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
 
-from ..engine import multi_execution_tuner
-from ..engine import oracle as oracle_module
-from ..engine import trial as trial_lib
+from keras_tuner.api_export import keras_tuner_export
+from keras_tuner.engine import oracle as oracle_module
+from keras_tuner.engine import trial as trial_module
+from keras_tuner.engine import tuner as tuner_module
 
 
+@keras_tuner_export("keras_tuner.oracles.RandomSearchOracle")
 class RandomSearchOracle(oracle_module.Oracle):
     """Random search oracle.
 
     Args:
-        objective: A string or `keras_tuner.Objective` instance. If a string,
-            the direction of the optimization (min or max) will be inferred.
+        objective: A string, `keras_tuner.Objective` instance, or a list of
+            `keras_tuner.Objective`s and strings. If a string, the direction of
+            the optimization (min or max) will be inferred. If a list of
+            `keras_tuner.Objective`, we will minimize the sum of all the
+            objectives to minimize subtracting the sum of all the objectives to
+            maximize. The `objective` argument is optional when
+            `Tuner.run_trial()` or `HyperModel.fit()` returns a single float as
+            the objective to minimize.
         max_trials: Integer, the total number of trials (model configurations)
             to test at most. Note that the oracle may interrupt the search
             before `max_trial` models have been tested if the search space has
-            been exhausted.
+            been exhausted. Defaults to 10.
         seed: Optional integer, the random seed.
         hyperparameters: Optional `HyperParameters` instance. Can be used to
             override (or register in advance) hyperparameters in the search
@@ -45,24 +50,34 @@ class RandomSearchOracle(oracle_module.Oracle):
         allow_new_entries: Boolean, whether the hypermodel is allowed to
             request hyperparameter entries not listed in `hyperparameters`.
             Defaults to True.
+        max_retries_per_trial: Integer. Defaults to 0. The maximum number of
+            times to retry a `Trial` if the trial crashed or the results are invalid.
+        max_consecutive_failed_trials: Integer. Defaults to 3. The maximum number of
+            consecutive failed `Trial`s. When this number is reached, the search
+            will be stopped. A `Trial` is marked as failed when none of the
+            retries succeeded.
     """
 
     def __init__(
         self,
-        objective,
-        max_trials,
+        objective=None,
+        max_trials=10,
         seed=None,
         hyperparameters=None,
         allow_new_entries=True,
         tune_new_entries=True,
+        max_retries_per_trial=0,
+        max_consecutive_failed_trials=3,
     ):
-        super(RandomSearchOracle, self).__init__(
+        super().__init__(
             objective=objective,
             max_trials=max_trials,
             hyperparameters=hyperparameters,
             tune_new_entries=tune_new_entries,
             allow_new_entries=allow_new_entries,
             seed=seed,
+            max_retries_per_trial=max_retries_per_trial,
+            max_consecutive_failed_trials=max_consecutive_failed_trials,
         )
 
     def populate_space(self, trial_id):
@@ -74,27 +89,38 @@ class RandomSearchOracle(oracle_module.Oracle):
         Returns:
             A dictionary with keys "values" and "status", where "values" is
             a mapping of parameter names to suggested values, and "status"
-            is the TrialStatus that should be returned for this trial (one
-            of "RUNNING", "IDLE", or "STOPPED").
+            should be one of "RUNNING" (the trial can start normally), "IDLE"
+            (the oracle is waiting on something and cannot create a trial), or
+            "STOPPED" (the oracle has finished searching and no new trial should
+            be created).
         """
         values = self._random_values()
         if values is None:
-            return {"status": trial_lib.TrialStatus.STOPPED, "values": None}
-        return {"status": trial_lib.TrialStatus.RUNNING, "values": values}
+            return {"status": trial_module.TrialStatus.STOPPED, "values": None}
+        return {"status": trial_module.TrialStatus.RUNNING, "values": values}
 
 
-class RandomSearch(multi_execution_tuner.MultiExecutionTuner):
+@keras_tuner_export(["keras_tuner.RandomSearch", "keras_tuner.tuners.RandomSearch"])
+class RandomSearch(tuner_module.Tuner):
     """Random search tuner.
 
     Args:
-        hypermodel: A `HyperModel` instance (or callable that takes
-            hyperparameters and returns a Model instance).
-        objective: A string or `keras_tuner.Objective` instance. If a string,
-            the direction of the optimization (min or max) will be inferred.
+        hypermodel: Instance of `HyperModel` class (or callable that takes
+            hyperparameters and returns a Model instance). It is optional when
+            `Tuner.run_trial()` is overriden and does not use
+            `self.hypermodel`.
+        objective: A string, `keras_tuner.Objective` instance, or a list of
+            `keras_tuner.Objective`s and strings. If a string, the direction of
+            the optimization (min or max) will be inferred. If a list of
+            `keras_tuner.Objective`, we will minimize the sum of all the
+            objectives to minimize subtracting the sum of all the objectives to
+            maximize. The `objective` argument is optional when
+            `Tuner.run_trial()` or `HyperModel.fit()` returns a single float as
+            the objective to minimize.
         max_trials: Integer, the total number of trials (model configurations)
             to test at most. Note that the oracle may interrupt the search
             before `max_trial` models have been tested if the search space has
-            been exhausted.
+            been exhausted. Defaults to 10.
         seed: Optional integer, the random seed.
         hyperparameters: Optional `HyperParameters` instance. Can be used to
             override (or register in advance) hyperparameters in the search
@@ -107,19 +133,27 @@ class RandomSearch(multi_execution_tuner.MultiExecutionTuner):
         allow_new_entries: Boolean, whether the hypermodel is allowed to
             request hyperparameter entries not listed in `hyperparameters`.
             Defaults to True.
+        max_retries_per_trial: Integer. Defaults to 0. The maximum number of
+            times to retry a `Trial` if the trial crashed or the results are invalid.
+        max_consecutive_failed_trials: Integer. Defaults to 3. The maximum number of
+            consecutive failed `Trial`s. When this number is reached, the search
+            will be stopped. A `Trial` is marked as failed when none of the
+            retries succeeded.
         **kwargs: Keyword arguments relevant to all `Tuner` subclasses.
             Please see the docstring for `Tuner`.
     """
 
     def __init__(
         self,
-        hypermodel,
-        objective,
-        max_trials,
+        hypermodel=None,
+        objective=None,
+        max_trials=10,
         seed=None,
         hyperparameters=None,
         tune_new_entries=True,
         allow_new_entries=True,
+        max_retries_per_trial=0,
+        max_consecutive_failed_trials=3,
         **kwargs
     ):
         self.seed = seed
@@ -130,5 +164,7 @@ class RandomSearch(multi_execution_tuner.MultiExecutionTuner):
             hyperparameters=hyperparameters,
             tune_new_entries=tune_new_entries,
             allow_new_entries=allow_new_entries,
+            max_retries_per_trial=max_retries_per_trial,
+            max_consecutive_failed_trials=max_consecutive_failed_trials,
         )
-        super(RandomSearch, self).__init__(oracle, hypermodel, **kwargs)
+        super().__init__(oracle, hypermodel, **kwargs)
