@@ -1,4 +1,4 @@
-# Copyright 2019 The Keras Tuner Authors
+# Copyright 2019 The KerasTuner Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,31 +13,40 @@
 # limitations under the License.
 """Tuner for Scikit-learn Models."""
 import collections
+import inspect
 import os
 import pickle
 import warnings
 
 import numpy as np
-import pandas as pd
 import tensorflow as tf
 
+try:
+    import pandas as pd  # pytype: disable=import-error
+except ImportError:
+    pd = None
+
 try:
     import sklearn  # pytype: disable=import-error
+    import sklearn.model_selection
+    import sklearn.pipeline
 except ImportError:
     sklearn = None
 
-from ..engine import base_tuner
+from keras_tuner.api_export import keras_tuner_export
+from keras_tuner.engine import base_tuner
 
 
 def split_data(data, indices):
     if isinstance(data, np.ndarray):
         return data[indices]
-    elif isinstance(data, pd.DataFrame):
+    elif pd is not None and isinstance(data, pd.DataFrame):
         return data.iloc[indices]
     else:
         raise TypeError()
 
 
+@keras_tuner_export(["keras_tuner.SklearnTuner", "keras_tuner.tuners.SklearnTuner"])
 class SklearnTuner(base_tuner.BaseTuner):
     """Tuner for Scikit-learn Models.
 
@@ -46,7 +55,7 @@ class SklearnTuner(base_tuner.BaseTuner):
     Examples:
 
     ```python
-    import keras_tuner as kt
+    import keras_tuner
     from sklearn import ensemble
     from sklearn import datasets
     from sklearn import linear_model
@@ -64,9 +73,9 @@ class SklearnTuner(base_tuner.BaseTuner):
             alpha=hp.Float('alpha', 1e-3, 1, sampling='log'))
       return model
 
-    tuner = kt.tuners.SklearnTuner(
-        oracle=kt.oracles.BayesianOptimizationOracle(
-            objective=kt.Objective('score', 'max'),
+    tuner = keras_tuner.tuners.SklearnTuner(
+        oracle=keras_tuner.oracles.BayesianOptimizationOracle(
+            objective=keras_tuner.Objective('score', 'max'),
             max_trials=10),
         hypermodel=build_model,
         scoring=metrics.make_scorer(metrics.accuracy_score),
@@ -145,7 +154,6 @@ class SklearnTuner(base_tuner.BaseTuner):
         return super().search(X, y, sample_weight=sample_weight, groups=groups)
 
     def run_trial(self, trial, X, y, sample_weight=None, groups=None):
-
         metrics = collections.defaultdict(list)
         # For cross-validation methods that expect a `groups` argument.
         cv_kwargs = {"groups": groups} if groups is not None else {}
@@ -160,7 +168,9 @@ class SklearnTuner(base_tuner.BaseTuner):
             )
 
             model = self.hypermodel.build(trial.hyperparameters)
-            if isinstance(model, sklearn.pipeline.Pipeline):
+
+            supports_sw = "sample_weight" in inspect.getfullargspec(model.fit).args
+            if isinstance(model, sklearn.pipeline.Pipeline) or not supports_sw:
                 model.fit(X_train, y_train)
             else:
                 model.fit(X_train, y_train, sample_weight=sample_weight_train)
@@ -185,9 +195,8 @@ class SklearnTuner(base_tuner.BaseTuner):
                     )
                     metrics[metric.__name__].append(result)
 
-        trial_metrics = {name: np.mean(values) for name, values in metrics.items()}
-        self.oracle.update_trial(trial.trial_id, trial_metrics)
         self.save_model(trial.trial_id, model)
+        return {name: np.mean(values) for name, values in metrics.items()}
 
     def save_model(self, trial_id, model, step=0):
         fname = os.path.join(self.get_trial_dir(trial_id), "model.pickle")
