@@ -13,6 +13,7 @@
 # limitations under the License.
 """KerasTuner utilities."""
 
+
 import warnings
 
 import IPython
@@ -22,11 +23,7 @@ from packaging.version import parse
 # Check if we are in a ipython/colab environement
 try:
     class_name = IPython.get_ipython().__class__.__name__
-    if "Terminal" in class_name:
-        IS_NOTEBOOK = False
-    else:
-        IS_NOTEBOOK = True
-
+    IS_NOTEBOOK = "Terminal" not in class_name
 except NameError:
     IS_NOTEBOOK = False
 
@@ -65,3 +62,29 @@ def check_tf_version():
             "ok.",
             ImportWarning,
         )
+
+
+def serialize_keras_object(obj):
+    if hasattr(tf.keras.utils, "legacy"):
+        return tf.keras.utils.legacy.serialize_keras_object(obj)
+    else:
+        return tf.keras.utils.serialize_keras_object(obj)
+
+
+def deserialize_keras_object(config, module_objects=None, custom_objects=None):
+    if hasattr(tf.keras.utils, "legacy"):
+        return tf.keras.utils.legacy.deserialize_keras_object(
+            config, custom_objects, module_objects
+        )
+    else:
+        return tf.keras.utils.deserialize_keras_object(
+            config, custom_objects, module_objects
+        )
+
+
+def to_list(values):
+    if isinstance(values, list):
+        return values
+    if isinstance(values, tuple):
+        return list(values)
+    return [values]
